import random,time
import os,sys
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)

from swaggerjmx.convert import conversion
from swaggerjmx.settings import Settings as ST
import traceback



def swagger2jmeter(url,reportPath):

    #  swagger_url
    try:
        ST.swagger_url = url
        #  report_path
        ST.report_path = reportPath
        # 开始转换
        conversion()
        # print(os.path.abspath(os.path.dirname(__file__))+os.sep+'jmx'+os.sep)

        upperDir = reportPath
        # upperDir=os.path.abspath(os.path.dirname(__file__))+os.sep+'jmx'+os.sep

        for x,y,z in os.walk(upperDir):
            return upperDir+"".join(z)
    except Exception as e:
        print(traceback.format_exc())
        return "switch to jmeter script error"


if __name__=="__main__":
    pass
    # print(swagger2jmeter("http://192.168.xxx.xxx:43323/auth/v2/api-docs"))


import click
import os
from core.utils import mkdirAfterCheck,mkfileAfterCheck
"""
this is test framework base on pytest and pytest-bdd
try to improve the feeling of using test framework

try to improve:
1. require user to follow the rule of this framework, enhance internal-understander in the test team
2. improve the way of parameterize 
3. using common used library to faster the script developing
4. re-org the test case, test suite, log, parameter, comander
5. union the report data and generate report
6. 

"""


@click.command()
@click.option('--name',default='kframe_proj',help='input name of the this project, default is kframe')
@click.option('--top_dir',default='.',help='input top directory of the this project, default is current directory')
def kframe_startProject(name,top_dir):
    cur_dir=os.path.abspath(top_dir)
    proj_dir=cur_dir+os.sep+name
    log_dir=proj_dir+os.sep+'logs'
    data_dir=proj_dir+os.sep+'datas'
    case_dir=proj_dir+os.sep+'cases'
    suite_dir=proj_dir+os.sep+'suites'
    report_dir=proj_dir+os.sep+'reports'
    config_dir=proj_dir+os.sep+'configs'
    mkdirAfterCheck(proj_dir)
    mkdirAfterCheck(log_dir)
    mkdirAfterCheck(data_dir)
    mkdirAfterCheck(case_dir)
    mkdirAfterCheck(suite_dir)
    mkdirAfterCheck(report_dir)
    mkdirAfterCheck(config_dir)

    print('init project done....')

@click.command()
@click.option('--name',default='test_case1',help='input name of testcase, default is test_case1')
@click.option('--top_dir',default='.',help='input top directory of the this project, default is current directory')
def kframe_createCase(name,default):
    cur_dir=os.path.abspath(top_dir)
    proj_dir=cur_dir+os.sep+name
    data_dir=proj_dir+os.sep+'datas'
    case_dir=proj_dir+os.sep+'cases'
    suite_dir=proj_dir+os.sep+'suites'
    config_dir=proj_dir+os.sep+'configs'
    mkfileAfterCheck(path)

    print('test case created....')


kframe_startProject()


import random,time
import os,sys
import platform as pf
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
from time import sleep
import shutil
from datetime import datetime

from loguru import logger
from pywebio.input import FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox,radio,actions
from pywebio.output import close_popup, output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row,put_processbar,set_processbar,put_error,put_warning,toast,put_grid,put_button,put_table,use_scope,span,clear,remove
from pywebio import start_server,session,platform
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.jmeterTool.swagger2jmeter import swagger2jmeter
from core.jmeterTool.har2jmeter import har2jmeter
from core.xmind2excel import makeCase
from core.utils import CFacker,getDateTime,parseJmeterXml,getTimeStamp,getDateTime,formatStr2Timestamp,timeStampStr2FormatTime,timeComparedBySeconds,jsonPrettyOutput
from core.mqttUtil import NormalMqttGetter,NormalMqttSender
from core.kafkaUtil import general_sender,continue_orderMsg,general_orderMsg,general_orderMsgWithFilter,kafkaFetchServerWithFilter,kafkaFetchServer
from functools import partial
from multiprocessing import Process
import decimal,websockets,asyncio
import json
from functools import partial

import pywebio.output as output
import pywebio.input as inputs
import pywebio.pin as pin
from pywebio.session import hold


@use_scope('content',clear=True)
def toolGeter():
    session.set_env(title='testToolKit')
    clear('content')
    select_type = select("选择你要做的操作:",["时间戳相关","数据构造","json格式化验证"])
    if select_type=="时间戳相关":
        timeStampGetter()
    elif select_type=="数据构造":
        myFackData()
    elif select_type=="json格式化验证":
        jsonFormater()

def jsonFormater():
    output.put_markdown("## Json格式化：")
    pin.put_textarea(name='oriJson',rows=10,help_text='请输入Json')
    def getValue():
        oriStr=pin.pin.oriJson
        output.put_text(jsonPrettyOutput(oriStr))
    output.put_button(label='格式化', onclick=lambda :getValue())



def timeStampGetter():
    output.set_scope('time' ,if_exist='remove')
    output.put_markdown("# 当前时间：",scope='time')
    output.put_text("- "+str(getDateTime()),scope='time')
    def refreshTime():
        output.put_text("- "+str(getDateTime()),scope='time')
    output.put_button(label='刷新时间', onclick=lambda :refreshTime())

    output.set_scope('time2',if_exist='remove')
    output.put_markdown("# 当前时间戳：",scope='time2')
    output.put_text("- "+str(getTimeStamp()),scope='time2')
    def refreshTimestamp():
        output.put_text("- "+str(getTimeStamp()),scope='time2')
    output.put_button(label='刷新时间戳', onclick=lambda :refreshTimestamp())

    output.set_scope('time3',if_exist='remove')
    output.put_markdown("## 时间转时间戳：",scope='time3')
    pin.put_input(name='curTime',label='请输入格式化时间',value="请输入格式化时间：2022-01-01 23:59:59.999",scope='time3')
    def time2stamp():
        curTime=pin.pin.curTime
        output.put_text(formatStr2Timestamp(curTime),scope='time3')
    output.put_button(label='确定', onclick=lambda :time2stamp())

    output.set_scope('time4',if_exist='remove')
    output.put_markdown("## 时间戳转时间：",scope='time4')
    pin.put_input(name='timeStamp',label='请输入时间戳',value="请输入13位时间戳，不够时后三位可写位0",scope='time4')
    def stamp2time():
        timeStamp=pin.pin.timeStamp
        output.put_text(timeStampStr2FormatTime(timeStamp),scope='time4')
    output.put_button(label='确定', onclick=lambda :stamp2time())




class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, decimal.Decimal):
            return float(o)
        super(DecimalEncoder, self).default(o)

@use_scope('content',clear=True)
def myFackData():
    '''
    generate the fake data by using this app, when you test
    :return:
    '''
    
    try:
        session.set_env(title='testToolKit')
        clear('content')
        all_options={
        "city_suffix":"市，县",
        "country":"国家",
        "country_code":"国家编码",
        "district":"区",
        "latitude":"地理坐标(纬度)",
        "longitude":"地理坐标(经度)",
        "postcode":"邮编",
        "province":"省份 (zh_TW没有此方法)",
        "address":"详细地址",
        "street_address":"街道地址",
        "street_name":"街道名",
        "street_suffix":"街、路",
        "ssn":"生成身份证号",
        "bs":"随机公司服务名",
        "company":"随机公司名（长）",
        "company_prefix":"随机公司名（短）",
        "company_suffix":"公司性质",
        "credit_card_expire":"随机信用卡到期日",
        "credit_card_full":"生成完整信用卡信息",
        "credit_card_number":"信用卡号",
        "credit_card_provider":"信用卡类型",
        "credit_card_security_code":"信用卡安全码",
        "job":"随机职位",
        "first_name":"名",
        "first_name_female":"女性名",
        "first_name_male":"男性名",
        "first_romanized_name":"罗马名",
        "last_name":"姓",
        "last_name_female":"女姓",
        "last_name_male":"男姓",
        "last_romanized_name":"随机",
        "name":"随机生成全名",
        "name_female":"男性全名",
        "name_male":"女性全名",
        "romanized_name":"罗马名",
        "msisdn":"移动台国际用户识别码，即移动用户的ISDN号码",
        "phone_number":"随机生成手机号",
        "phonenumber_prefix":"随机生成手机号段",
        "ascii_company_email":"随机ASCII公司邮箱名",
        "ascii_email":"随机ASCII邮箱",
        "ascii_free_email":"二进制免费邮件",
        "ascii_safe_email":"二进制安全邮件",
        "company_email":"公司邮件",
        "email":"电子邮件",
        "free_email":"免费电子邮件",
        "free_email_domain":"免费电子邮件域名",
        "safe_email":"安全邮箱",
        "domain_name":"生成域名",
        "domain_word":"域词(即，不包含后缀)",
        "ipv4":"随机IP4地址",
        "ipv6":"随机IP6地址",
        "mac_address":"随机MAC地址",
        "tld":"网址域名后缀(.com,.net.cn,等等，不包括.)",
        "uri":"随机URI地址",
        "uri_extension":"网址文件后缀",
        "uri_page":"网址文件（不包含后缀）",
        "uri_path":"网址文件路径（不包含文件名）",
        "url":"随机URL地址",
        "user_name":"随机用户名",
        "image_url":"随机URL地址",
        "chrome":"随机生成Chrome的浏览器user_agent信息",
        "firefox":"随机生成FireFox的浏览器user_agent信息",
        "internet_explorer":"随机生成IE的浏览器user_agent信息",
        "opera":"随机生成Opera的浏览器user_agent信息",
        "safari":"随机生成Safari的浏览器user_agent信息",
        "linux_platform_token":"随机Linux信息",
        "user_agent":"随机user_agent信息",
        "file_extension":"随机文件扩展名",
        "file_name":"随机文件名（包含扩展名，不包含路径）",
        "file_path":"随机文件路径（包含文件名，扩展名）",
        "mime_type":"随机mime Type",
        "numerify":"三位随机数字",
        "random_digit":"0~9随机数",
        "random_digit_not_null":"1~9的随机数",
        "random_int":"随机数字，默认0~9999，可以通过设置min,max来设置",
        "random_number":"随机数字，参数digits设置生成的数字位数",
        "pyfloat":"left_digits=5 #生成的整数位数, right_digits=2 #生成的小数位数, positive=True #是否只有正数",
        "pyint":"随机Int数字（参考random_int=参数）",
        "pydecimal":"随机Decimal数字（参考pyfloat参数）",
        "pystr":"随机字符串",
        "random_element":"随机字母",
        "random_letter":"随机字母",
        "paragraph":"随机生成一个段落",
        "paragraphs":"随机生成多个段落，通过参数nb来控制段落数，返回数组",
        "sentence":"随机生成一句话",
        "sentences":"随机生成多句话，与段落类似",
        "text":"随机生成一篇文章（不要幻想着人工智能了，至今没完全看懂一句话是什么意思）",
        "word":"随机生成词语",
        "words":"随机生成多个词语，用法与段落，句子，类似",
        "binary":"随机生成二进制编码",
        "boolean":"True/False",
        "language_code":"随机生成两位语言编码",
        "locale":"随机生成语言/国际 信息",
        "md5":"随机生成MD5",
        "null_boolean":"NULL/True/False",
        "password":"随机生成密码,可选参数：length：密码长度；special_chars：是否能使用特殊字符；digits：是否包含数字；upper_case：是否包含大写字母；lower_case：是否包含小写字母",
        "sha1":"随机SHA1",
        "sha256":"随机SHA256",
        "uuid4":"随机UUID",
        "am_pm":"AM/PM",
        "century":"随机世纪",
        "date":"随机日期",
        "date_between":"随机生成指定范围内日期，参数：start_date，end_date取值：具体日期或者today,-30d,-30y类似",
        "date_between_dates":"随机生成指定范围内日期，用法同上",
        "date_object":"随机生产从1970-1-1到指定日期的随机日期。",
        "date_this_month":"当月",
        "date_this_year":"当年",
        "date_time":"随机生成指定时间（1970年1月1日至今）",
        "date_time_ad":"生成公元1年到现在的随机时间",
        "date_time_between":"用法同dates",
        "future_date":"未来日期",
        "future_datetime":"未来时间",
        "month":"随机月份",
        "month_name":"随机月份（英文）",
        "past_date":"随机生成已经过去的日期",
        "past_datetime":"随机生成已经过去的时间",
        "time":"随机24小时时间",
        "time_object":"随机24小时时间，time对象",
        "time_series":"随机TimeSeries对象",
        "timezone":"随机时区",
        "unix_time":"随机Unix时间",
        "year":"随机年份",
        "profile":"随机生成档案信息",
        "simple_profile":"随机生成简单档案信息",
        "currency_code":"货币编码",
        "color_name":"随机颜色名",
        "hex_color":"随机HEX颜色",
        "rgb_color":"随机RGB颜色",
        "safe_color_name":"随机安全色名",
        "safe_hex_color":"随机安全HEX颜色",
        "isbn10":"随机ISBN(10位)",
        "isbn13":"随机ISBN(13位)",
        "lexify":"替换所有问号?带有随机字母的事件"
        }
        
        num=inputs.input("生成几组数据，默认是1组", type=NUMBER, value=1)
        # print(f'num is {num}')
        myformat=inputs.radio(label='选择生成数据格式：支持csv,json',options=['csv','json'],value=['json'])
        # print(f'format is {myformat}')
        
        choose=inputs.checkbox(label='从下列选项中，选择你想生成的数据：',options=all_options.values())
        
        # put_row([input('自定义键值：'),checkbox(options=[''])])
        allDict={}
        for i in range(0,num):
            restDict={}
            for one in choose:
                funcName=list(all_options.keys())[list(all_options.values()).index(one)]
                restDict[one]=CFacker().get_it(funcName)
            allDict[i]=restDict
        
        # put_text(allDict)
        
        if myformat=='json':
            
            output.put_code(json.dumps(allDict,cls=DecimalEncoder, indent=4,ensure_ascii=False), language='json',rows=20) 
        elif myformat=='csv':
            
            outStr=""
            for k,v in allDict.items():
                for vv in v.values():
                    # print(f"vv {vv}")
                    outStr=outStr+str(vv)+","
                outStr=outStr+"\n"
                
            output.put_text(outStr) 
    except Exception as e:
        output.popup(title="error",content=output.put_text(e))
        clear('content')


def kafkaSender():
    output.put_markdown("## 请输入kafka连接信息")
    pin.put_input(name='host',label='主机：端口')
    pin.put_input(name='topic',label='主题')
    pin.put_textarea(name='msg',label='消息',rows=10)
    pin.put_input(name='interval',label='发送间隔',value=0)
    def getValueAndCall():
        host=pin.pin.host
        topic=pin.pin.topic
        msg=pin.pin.msg
        interval=int(pin.pin.interval)
        
        if interval==0:
            general_sender(topic=topic,serverAndPort=host,message=msg)
            output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
        elif interval>0:
            while True:
                general_sender(topic=topic,serverAndPort=host,message=msg)
                output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
                sleep(int(interval))

    output.put_button(label='提交', onclick=lambda :getValueAndCall())


def kafkaGeter():
    put_text("json过滤如：data.deviceId")
    data = input_group("kafka连接配置", [
        inputs.input("kafka topic，必填", name="topic",value='testTopic'),
        inputs.input("kafka 地址，如ip:port，必填", name="address",value='192.168.xxx.xxx:9092'),
        inputs.input("过滤方式，仅支持填json或regx，非必填", name="filter",value='json'),
        inputs.input("过滤表达式，json使用jmeshpath方式，regx采用abc(.*)bbb的方式，非必填", name="pattern",value="payload.name"),
        inputs.input("过滤后比对关键字，过滤后的值是否等于输入的值，非必填", name="key",value="status"),
    ])
            
    for one in continue_orderMsg(data['topic'], data['address'], data['filter'], data['pattern'], data['key']):
        for a in one:
            if a is not None:
                put_text(f"{getDateTime()} : {data['topic']} --> {a}")

class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, decimal.Decimal):
            return float(o)
        super(DecimalEncoder, self).default(o)


@use_scope('content',clear=True)
def kafkaListener():
    '''
    to send kafka message or listener the kafka topic
    
    '''
    try:
        session.set_env(title='testTools')
        clear('content')
        select_type = select("选择kafka操作:",["kafka发送消息","kafka持续接收消息","kafka消息转发"])

        if select_type=="kafka发送消息":
            kafkaSender()
        elif select_type=="kafka持续接收消息":
            kafkaGeter()
        elif select_type=="kafka消息转发":
            kafkaTransfer()
    except Exception as e:
        output.popup(title="error",content=put_text(e))
        clear('content')







def mqttSender():
    output.put_markdown("## 请输入mqtt连接信息")
    pin.put_input(name='host',label='mqtt主机，必填')
    pin.put_input(name='port',label='mqtt端口，必填（整数）')
    pin.put_input(name='topic',label='mqtt topic，必填')
    pin.put_input(name='user',label='mqtt 用户')
    pin.put_input(name='passwd',label='mqtt 密码')
    # pin.put_input(name='filter',label='mqtt 消息过滤字段')
    pin.put_textarea(name='msg',label='消息',rows=10)
    pin.put_input(name='interval',label='发送间隔',value=0)
    def getValueAndCall():
        host=pin.pin.host
        port=int(pin.pin.port)
        topic=pin.pin.topic
        user=pin.pin.user
        passwd=pin.pin.passwd
        msg=pin.pin.msg
        interval=int(pin.pin.interval)
        # myfilter=pin.pin.filter
        
        if interval==0:
            NormalMqttSender(host=host, port=port, topic=topic,user=user,passwd=passwd).getClient(msg)
            output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
        elif interval>0:
            while True:
                NormalMqttSender(host=host, port=port, topic=topic,user=user,passwd=passwd).getClient(msg)
                output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
                sleep(interval)
    output.put_button(label='提交', onclick=lambda :getValueAndCall())


def mqttGeter():
    data = input_group("mqtt信息",[
        inputs.input("mqtt主机，必填", name="host"),
        inputs.input("mqtt端口，必填（整数）", name="port"),
        inputs.input("mqtt topic，必填",name="topic"),
        inputs.input("mqtt用户", name="user"),
        inputs.input("mqtt密码", name="passwd"),
        inputs.input("消息包含", name="filter"),
                ])
            # put_button(label=f"{data['host']}:{data['port']}-{data['topic']}-{data['user']}:{data['passwd']}", onclick=NormalMqttGetter().close(client))
    if data['filter']=="" or data['filter']==None:
        NormalMqttGetter(host=data['host'], port=int(data['port']), topic=data['topic'],user=data['user'],passwd=data['passwd']).getClient(partial(filterPrint, tarStr=data['filter']))
    NormalMqttGetter(host=data['host'], port=int(data['port']), topic=data['topic'],user=data['user'],passwd=data['passwd']).getClient(noFilterPrint)




@use_scope('content',clear=True)
def mqttListener():
    try:
        session.set_env(title='testTools')
        clear('content')

        # select_type = select("选择mqtt服务:",["自定义发送服务","自定义接收服务","本地固定服务(待定)"])
        select_type = select("选择mqtt服务:",["自定义发送服务","自定义接收服务","本地固定服务(待定)","MQTT消息转发"])
        if select_type=="自定义接收服务":
            mqttGeter()
        elif select_type=="自定义发送服务":
            mqttSender()
        elif select_type == "本地固定服务(待定)":
            put_text('未暴露')
        elif select_type=="MQTT消息转发":
            mqttTransfer()
        
    except Exception as e:
        output.popup(title="error",content=put_text(e))
        
@use_scope('content',clear=True)
def kafkaTransfer():
    try:
        output.put_markdown("## 请输入需要转发的kafka信息")
        pin.put_input(name='host',label='源主机：端口*')
        pin.put_input(name='topic',label='源主题*')
        pin.put_input(name='user',label='源用户,非必填')
        pin.put_input(name='passwd',label='源密码,非必填')
        output.put_markdown("-----------------------------")
        pin.put_input(name='host2',label='目的主机：端口*')
        pin.put_input(name='topic2',label='目的主题*')
        pin.put_input(name='user2',label='目的用户,非必填')
        pin.put_input(name='passwd2',label='目的密码,非必填')
        pin.put_input(name='sec2',label='持续转发多长时间，单位秒*')

        def getValueAndCall():
            host=pin.pin.host
            topic=pin.pin.topic
            user=pin.pin.user
            passwd=pin.pin.passwd

            host2=pin.pin.host2
            topic2=pin.pin.topic2
            user2=pin.pin.user2
            passwd2=pin.pin.passwd2
            sec2=int(pin.pin.sec2)

            start=datetime.now()
            for one in continue_orderMsg(topic=topic, serverAndPort=host,flag="", pattern="", key=""):
                for msg in one:
                    print(timeComparedBySeconds(start, datetime.now()), sec2)
                    if timeComparedBySeconds(start, datetime.now()) < sec2:
                        output.toast(content=msg)
                        general_sender(topic=topic2,serverAndPort=host2,message=msg)
                    else:
                        exit(0)


        output.put_button(label='提交', onclick=lambda :getValueAndCall())
    except Exception as e:
        print(e)
        exit()
    finally:
        exit()




from functools import partial
def filterPrint(oriStr,tarStr):
    if tarStr in oriStr:
        put_text(getDateTime()+" "+oriStr)

def filterSender(oriStr,tarStr,origStr,destStr,host,port,user,passwd,topic):
    if tarStr in oriStr:
        newStr=oriStr.replace(origStr,destStr)
        output.toast(content=newStr)
        NormalMqttSender(host=host,port=port,user=user,passwd=passwd,topic=topic).getClient(msg=newStr)
        
def filterSenderWithTime(oriStr,tarStr,origStr,destStr,host,port,user,passwd,topic,time_sec):
    # counter=0
    start=datetime.now()
    while timeComparedBySeconds(start, datetime.now())<time_sec:
        if tarStr in oriStr:
            newStr=oriStr.replace(origStr,destStr)
            output.toast(content=newStr)
            NormalMqttSender(host=host,port=port,user=user,passwd=passwd,topic=topic).getClient(msg=newStr)
            time.sleep(1)
            # counter=counter+1
    else:
        exit(0)

def noFilterPrint(oriStr):
    put_text(getDateTime()+" "+oriStr)

@use_scope('content',clear=True)
def mqttTransfer():
    try:
        output.put_markdown("## 请输入需要转发的，源MQTT信息")
        pin.put_input(name='host',label='源主机*')
        pin.put_input(name='port',label="源端口*")
        pin.put_input(name='topic',label='源主题*')
        pin.put_input(name='user',label='源用户*')
        pin.put_input(name='passwd',label='源密码*')
        pin.put_input(name='filter',label='包含次字符串才被转发',value=None)
        output.put_markdown("## 请输入需要转发到的，目的MQTT信息")
        pin.put_input(name='host2',label='目的主机*')
        pin.put_input(name='port2',label="目的端口*")
        pin.put_input(name='topic2',label='目的主题*')
        pin.put_input(name='user2',label='目的用户*')
        pin.put_input(name='passwd2',label='目的密码*')
        pin.put_input(name='sec2',label='持续转发多长时间，单位秒*')
        pin.put_input(name='origStr',label='转发时查找字符串',value=None)
        pin.put_input(name='destStr',label='转发时替换字符串',value=None)

        def getValueAndCall():
            host=pin.pin.host
            port=int(pin.pin.port)
            topic=pin.pin.topic
            user=pin.pin.user
            passwd=pin.pin.passwd
            myfilter=pin.pin.filter

            host2=pin.pin.host2
            port2=int(pin.pin.port2)
            topic2=pin.pin.topic2
            user2=pin.pin.user2
            passwd2=pin.pin.passwd2
            sec2=int(pin.pin.sec2)
            myOrig=pin.pin.origStr
            myDest=pin.pin.destStr

            NormalMqttGetter(host=host, port=port, topic=topic,user=user,passwd=passwd).getClient(partial(filterSenderWithTime,tarStr=myfilter,origStr=myOrig,destStr=myDest,host=host2,port=port2,user=user2,passwd=passwd2,topic=topic2,time_sec=sec2))

        output.put_button(label='转发', onclick=lambda :getValueAndCall())
        
    except Exception as e:
        print(e)
        exit()
    finally:
        exit()


if __name__ == '__main__':
    start_server(mqttListener,port=8999,debug=True,cdn=False,auto_open_webbrowser=True)





import random,time
import os,sys
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
from time import sleep
from loguru import logger
from pywebio.input import input, FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox,radio
from pywebio.output import close_popup, output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row,put_grid,span,put_table,put_button
from pywebio import start_server,session,platform
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.jmeterTool.swagger2jmeter import swagger2jmeter
from core.jmeterTool.har2jmeter import har2jmeter
from core.xmind2excel import makeCase
from core.utils import CFacker,getDateTime
from core.mqttUtil import NormalMqttGetter
from core.kafkaUtil import general_sender,continue_orderMsg,general_orderMsg,general_orderMsgWithFilter,kafkaFetchServerWithFilter,kafkaFetchServer
from functools import partial
from multiprocessing import Process
import decimal,websockets,asyncio
import json


put_table([
    ['操作', '内容'],
    ['xmind转excel', put_button(['xmind转excel'],onclick=abc)]
    # ['混沌测试-交互式', '<hr/>'],
    # ['混沌测试-直接输入(推荐)', put_buttons(['A', 'B'], onclick=put_text)],  
    # ['jmeter脚本生成', put_markdown('`Awesome PyWebIO!`')],
    # ['假数据构造', put_file('hello.text', b'hello world')],
    # ['kafka操作', put_table([['A', 'B'], ['C', 'D']])],
    # ['mqtt操作', put_table([['A', 'B'], ['C', 'D']])]
])

put_grid([
    [put_text('A'), put_text('B'), put_text('C')],
    [None, span(put_text('D'), col=2, row=1)],
    [put_text('E'), put_text('F'), put_text('G')],
], cell_width='100px', cell_height='100px')




import json
import yaml
import csv
import xlrd

def getFromJson(jsonFilePath):
    return json.load(open(jsonFilePath,'r'))

def getFromYaml(yamlFilePath):
    with open(yamlFilePath,'r') as f:
        return list(yaml.safe_load_all(f))

def getFromCsv(csvFilePath):
    data=[]
    with open(csvFilePath,'r') as f:
        reader=csv.reader(f)
        next(reader)
        for one in reader:
            data.append(one)
    return data

def getFromExcel(excelFilePath):
    data=[]
    book=xlrd.open_workbook(excelFilePath)
    sheet=book.sheet_by_index(0)
    for one in range(1,sheet.nrows):
        data.append(sheet.row_values(one))
    return data


#encoding:utf-8
def testDingGroupAlert(dingding_url,businessFunc=None):
    #钉钉告警接口，从钉钉api获取
    dingding_url="https://oapi.dingtalk.com/robot/send?access_token=e024afe8a1b7aff5896d5ba9axxxxxxx"
    #钉钉告警接口，发送模板
    def message_warp(request_url="requestUrl",response_message="responseMessage"):
        return {
            "msgtype": "markdown",
            "markdown": {
                "title":"业务异常通知",
                "text": "# 重要！！ 业务错误\n---\n## 发送请求：\n```\n"+request_url+"\n```\n## 返回消息：\n```\n"+response_message+"\n```\n"
            },
        }


    # #拨测业务数据接口调用
    # login_data={
    #     "phoneNum":"1834911xxxx",
    #     "password":"c03bf628eee907844xxxxxxxxxxxx",
    #     "businessCode":"xxxxxx-xxxxkit",
    #     "loginType":"ACCOUNT"
    # }
    # login_url="https://xxxxxx.xxxxxx.com/xxxxxx-xxxxkit/homekit/account/login"
    # login_header={"Content-Type":"application/json","grant_type":"password"}
    # client=requests.session()
    # res=client.post(url=login_url,headers=login_header,json=login_data)
    # login_token=json.loads(res.text)['data']['token']
    # login_userid=json.loads(res.text)['data']['userId']
    # if login_token=="":
    #     client.post(url=dingding_url,json=message_warp(login_url,res.text))
    #     exit()


    # query_url="https://xxxxxx.xxxxxx.com/xxxxxx-xxxxkit/homekit/device/query"
    # query_header={"Content-Type":"application/json","Authorization":login_token}
    # query_data={"accessToken":login_token,"applicationId":"3f61abxxxx","channelId":"abc_store","clientId":"0","currenTime":"16285xxxxxx","equipmentId":"","gatewayId":"50294D2246BC","userId":login_userid,"version":"1.0.0"}
    # res=client.post(url=query_url,headers=query_header,json=query_data)

    # result_flag=json.loads(res.text.encode('utf-8'))['success']
    # result_message=json.loads(res.text.encode('utf-8'))['message']
    # r=client.post(url=dingding_url,json=message_warp(query_url,res.text))
    # print(r.text)
    result_flag=businessFunc()
    if result_flag!=True or result_message!='成功':
        client.post(url=dingding_url,json=message_warp(query_url,res.text))
        exit()







def voiceTest():
    #语音播放，读取
    #coding:utf-8
    import pyttsx3
    import random
    from loguru import logger

    story='民国时期，胡八一祖父胡国华因为家道中落和自己吃喝嫖赌抽样样俱全而被迫去盗墓，却被一看风水老先生制止并于百年后传授一本秘书残卷十六字阴阳风水秘术，懂此书可知风水，可观天文，并以此可知隐藏在奇川大山中的琼楼古墓，此书历经磨难最终传到胡八一手上。'
    short_words=['革时期','上知','户山','热','得胡八一','之行','期间','Shirley杨']
    long_word='陈教授，以四人为主开始梦幻奇绝的探秘之旅。其中以四个大的墓葬为全文脉络，地点分别是沙漠中的精绝古城、陕西的龙岭迷窟、云南虫谷以及西藏昆仑山。故事中用两个小片段交代人物出场、故事背景以及关于盗墓的一些基本常识，高潮则开始于胡八一和胖子跟随考古队，利用秘书残卷中风水秘术来解读天下大川脉搏，寻找一处失落在地下的琼楼宝殿——精绝古城，从而揭开千年部族消失之谜，却在精绝古城中遭遇种种离奇，无论是尸香魔芋的怪异魔力还是关于先知的预言，最终考古队中只有四人从鬼洞中死里逃生。而在龙岭迷窟倒斗的时候，除了遭遇各种匪夷所思的诡异外，还意外地发现从精绝古城的鬼洞死里逃生出来的四个人（另外两个是Shirley杨和陈教授）都染上怪病，需要传说中埋藏在云南古滇国献王墓中的髦尘珠才能救各人性命，但髦尘珠所在地献王墓却隐藏在蛇河虫谷之中，胡八一等为解性命之忧不得已开始云南虫谷之旅。但是如何揭开髦尘珠的秘密，所有线索又指向雪域藏地，此时香港古董商为寻找魔国冰川水晶尸也意欲前往藏地古格王国遗址，从而又开始了一段昆仑神宫的探秘之行，而在异常离奇诡秘的地下世界中，历史的神秘面纱也正一层层地被揭开'
    english_chinese=['有些term不好translate','show off 自己的外语level','maybe是刚从国外return，还没有used to 全用中文','Jack，把你提交的代码merge request一下','review一下你的代码，那个工程rebuild一下']

    engine=pyttsx3.init()
    rate=engine.getProperty('rate')
    engine.setProperty('rate',166)
    volume = engine.getProperty('volume')   # 获取当前的音量 （默认值）(min=0 and max=1)
    #print (volume)                          # 打印当前音量（默认值）
    engine.setProperty('volume',1.0)    # 设置一个新的音量（0 < volume < 1）
    voices = engine.getProperty('voices')       # 获取当前的音色信息
    engine.setProperty('voice', voices[0].id)  # 改变中括号中的值,0为男性,1为女性
    engine.setProperty('voice','zh')             #将音色中修改音色的语句替换



def read_phash_07():
    words_list=[]
    f= open('C:\\Users\\oupeng\\Documents\\001-myfile\\项目\\语音转文字SDK测试\\语料.txt','r',encoding='utf-8')
    for one in f.readlines():
        words_list.append(one)
    logger.info(words_list)
    word_len=len(words_list)
    logger.info(word_len)
    sed_int=random.randint(1,10)
    logger.info(sed_int)
    words=""
    for one in range(0,sed_int):
        words=words+words_list[random.randint(0,word_len)]
    logger.info(words)
    engine.say(words)
    # engine.say("你好")
    engine.runAndWait()

def read_story_04():
    logger.info(story)
    engine.say(story)
    engine.runAndWait()

def read_shot_words_03():
    lenth=len(short_words)
    w=short_words[random.randint(0,lenth)]
    logger.info(w)
    engine.say(w)
    engine.runAndWait()

def read_en_ch_06():
    lenth2 = len(english_chinese)
    w = english_chinese[random.randint(0, lenth2)]
    logger.info(w)
    engine.say(w)
    engine.runAndWait()

def read_long_word():
    engine.say(long_word)
    engine.runAndWait()


def mp3gen(voiceStr='你好我来了',savePath="voice.mp3"):
    import pyttsx3
    import os
    tts = pyttsx3.init()
    tts.setProperty("rate", 50) #设置语速
    # tts.setProperty('voice', tts.getProperty('voice')[1])
    tts.setProperty('volume',1.0)    # 设置一个新的音量（0 < volume < 1）
    if os.path.exists(savePath):
        os.remove(savePath)
    tts.save_to_file(voiceStr, savePath)
    tts.runAndWait()


def namePass():
    import getpass
    # 自动读取当前用户的名称
    user = getpass.getuser()
    print("尊敬的",user)
    # 以不回显的方式,读取用户的输入
    passwd = getpass.getpass("请输入您的密码:")
    print("------------->华丽的分割线<----------------")
    print("您的密码为:", passwd)


if __name__=='__main__':
    # read_long_word()
    mp3gen("this is test 王者荣耀","voice.mp3")


#encoding:utf-8
import random,time
import os,sys,jsonpath
import platform as pf
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
from time import sleep
import shutil

from loguru import logger
from pywebio.input import FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox,radio,actions
from pywebio.output import close_popup, output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row,put_processbar,set_processbar,put_error,put_warning,toast,put_grid,put_button,put_table,use_scope,span,clear,remove
from pywebio import start_server,session,platform
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.jmeterTool.swagger2jmeter import swagger2jmeter
from core.jmeterTool.har2jmeter import har2jmeter
from core.xmind2excel import makeCase
from core.utils import CFacker,getDateTime,parseJmeterXml,getTimeStamp,getDateTime,formatStr2Timestamp,timeStampStr2FormatTime,jsonPrettyOutput
from core.mqttUtil import NormalMqttGetter,NormalMqttSender
from core.kafkaUtil import general_sender,continue_orderMsg,general_orderMsg,general_orderMsgWithFilter,kafkaFetchServerWithFilter,kafkaFetchServer
from functools import partial
from multiprocessing import Process
import decimal,websockets,asyncio
import json
from functools import partial

import pywebio.output as output
import pywebio.input as inputs
import pywebio.pin as pin
from pywebio.session import hold
import  traceback
from core.httpUtil import HttpOper



def modle2command_new(modle_str):
    try:
        jstr=json.loads(modle_str)#rawStr2)
        productIds=jsonpath.jsonpath(jstr, "$.standardFunctions[*].identifier")
        allcommands=jsonpath.jsonpath(jstr, "$.standardFunctions[*].commands")
        
        # print(productIds)
        # print(allcommands)

        # for a,b in zip(productIds,allcommands):
        #     print(a,b)

        if productIds.__len__()>=1:
            for productId,commands in zip(productIds,allcommands):
                # commands=jsonpath.jsonpath(jstr, "$.standardFunctions[*].commands[*]")
                if not commands:
                    continue
                else:
                    resTotal=[]
                    for command in commands:
                        # print('--'*10)
                        function_id=command['identifier']
                        command_id=""
                        command_value=[]
                        if command['inputs']!=[] :
                            # print(command['inputs'][0]['identifier'])
                            command_id=command['inputs'][0]['identifier']
                            # print(command['inputs'][0]['dataType'])
                            command_type=command['inputs'][0]['dataType']
                            
                            # if command_type=='enum' or command_type=='bool':
                            if command['inputs'][0].__contains__('specifications'):
                                values=command['inputs'][0]['specifications']
                                # print(values)
                                for value in values:
                                    # print(value['value'])
                                    command_value.append(value['value'])
                            elif command['inputs'][0].__contains__('specification'):
                                
                                values=command['inputs'][0]['specification']
                                if values.__contains__('min'):
                                    # print(values['min'])
                                    minNum=int(values['min'])
                                    command_value.append(minNum-1)
                                if values.__contains__('max'):
                                    # print(values['max'])
                                    maxNum=int(values['max'])
                                    command_value.append(maxNum+1)
                                # if values.__contains__('digit'):
                                    # print(values['digit'])
                                # if values.__contains__('step'):
                                    # print(values['step'])
                                if values.__contains__('accuracy'):
                                    # print(values['accuracy'])
                                    acc=int(values['accuracy'])
                                    command_value.append(round((minNum+maxNum)/2,acc))
                                    command_value.append(round((minNum+maxNum)/2,acc+1))
                                command_value.append((minNum+maxNum)//2)
                        # print(productId)
                        # print(function_id)
                        # print(command_id)
                        # print(command_value)

                        if command_value.__len__()>0:
                            for one in command_value:
                                tempLine=[]
                                tempLine.append(productId)
                                tempLine.append(function_id)
                                tempLine.append(command_id)
                                tempLine.append(one)
                                resTotal.append(tempLine)
                        else:
                                tempLine=[]
                                tempLine.append(productId)
                                tempLine.append(function_id)
                                resTotal.append(tempLine)
                    return resTotal
    except Exception as e:
        print(traceback.print_exc())



def commandConstruct(commandList,deviceId=123):
    totalCmds=""
    for one in commandList:
        if type(one)==list and one.__len__()==4:
            commandStr=f'''{{"cmd": {{"command": "{one[1]}","function": "{one[0]}","param":{{"{one[2]}":"{one[3]}"}} }},"deviceId": "{deviceId}"}}'''
            # return(commandStr)
            totalCmds=totalCmds+commandStr+"\n"
        elif type(one)==list and one.__len__()==2:
            commandStr=f'''{{"cmd": {{"command": "{one[1]}","function": "{one[0]}"}},"deviceId": "{deviceId}"}}'''
            # return(commandStr)
            totalCmds=totalCmds+commandStr+"\n"
    return totalCmds





@use_scope('content',clear=True)
def businessProcess():
    session.set_env(title='testToolKit')
    clear('content')
    select_type = select("选择你要做的操作:",["指令生成","指令生成发送","属性上报--","点位生成--"])
    if select_type=="指令生成":
        commandGenerator()
    elif select_type=="指令生成发送":
        commandGeneratorAndSend()




def modle2command(deviceId,modle_str):
    '''
    用法：
    将数据库xxxx表中字段model_content当做参数modle_str；会生成下发指令的集合
    如：
                                    {
                                "cmd": {
                                    "command": "door_status_set",
                                    "function": "door",
                                    "param":{"door_status_value":"['1', '0']"}
                                },
                                "deviceId": "123456"
                                }
    表示，可以下发的值包含['1', '0']
    '''

    totalCommand=""
    jstr=json.loads(modle_str)#rawStr2)
    productId=jsonpath.jsonpath(jstr, "$.standardFunctions[*].identifier")
    for num, p_id in enumerate(productId):
        totalCommand=totalCommand+"##############################"+"\n"
        commands=jsonpath.jsonpath(jstr, "$.standardFunctions["+str(num)+"].commands[*]")
        if commands:
            for command in commands:
                # print('--'*10)
                totalCommand=totalCommand+"-----------------------------"+"\n"
                # print(command['identifier'])
                function_id=command['identifier']
                # print(f"#######{command['inputs']}")
                command_value=[]
                if command['inputs']!=[] :
                    # print(command['inputs'][0]['identifier'])
                    command_id=command['inputs'][0]['identifier']
                    # print(command['inputs'][0]['dataType'])
                    command_type=command['inputs'][0]['dataType']
                    
                    if command['inputs'][0].__contains__('specifications'):
                        
                        values=command['inputs'][0]['specifications']
                        # print(values)
                        for value in values:
                            # print(value['value'])
                            command_value.append(value['value'])
                    elif command['inputs'][0].__contains__('specification'):
                        minNum=None
                        maxNum=None
                        acc=None
                        values=command['inputs'][0]['specification']
                        # print(f"##################################################{values}")
                        if values!="{}":
                            if values.__contains__('min'):
                                # print(values['min'])
                                minNum=int(values['min'])
                                command_value.append(minNum-1)
                            if values.__contains__('max'):
                                # print(values['max'])
                                maxNum=int(values['max'])
                                command_value.append(maxNum+1)
                            # if values.__contains__('digit'):
                                # print(values['digit'])
                            # if values.__contains__('step'):
                                # print(values['step'])
                            if values.__contains__('accuracy'):
                                # print(values['accuracy'])
                                acc=int(values['accuracy'])
                                command_value.append(round((minNum+maxNum)/2,acc))
                                command_value.append(round((minNum+maxNum)/2,acc+1))
                                command_value.append((minNum+maxNum)//2)
                # print(p_id)
                # print(function_id)
                # print(command_id)
                # print(command_value)
                commandStr=f'''{{"cmd": {{"command": "{function_id}","function": "{p_id}","param":{{"{command_id}":"{command_value}"}} }},"deviceId": "{deviceId}"}}
                '''
                totalCommand=totalCommand+commandStr+"\n"
            return totalCommand
            # print(totalCommand)

def commandGenerator():
    output.put_markdown("## 请输入设备信息和模型信息：")
    pin.put_input(name='deviceId',label='设备信息,设备id')
    pin.put_textarea(name='modleJson',label='模型信息,物模型定义Json字符串',rows=10)
    def getValueAndCall():
        deviceId=pin.pin.deviceId
        modleJson=pin.pin.modleJson
        commandList=[]
        for one in modleJson.split('\n'):
            commandList=commandList+modle2command_new(str(one))
        output.put_text(commandConstruct(deviceId=deviceId, commandList=commandList))
    output.put_button(label='提交', onclick=lambda :getValueAndCall())


def commandGeneratorAndSend():
    put_text("指令信息:")
    data = input_group("配置信息", [
        inputs.input("设备号，必填", name="deviceId",value='123456'),
        inputs.textarea("模型信息json，必填", name="model",value='json 字符串'),
        inputs.input("请求地址，必填", name="url",value='http://127.0.0.1:9999/device'),
        inputs.input("请求头，必填", name="header",value='{"Content-Type":"application/json"}'),
    ])
    # print(data['model'])
    # 旧代码
    commandStr=commandConstruct(deviceId=data['deviceId'], commandList=modle2command_new(data['model']))
    commandStrList=commandStr.split('\n')
    #兼容多个模型,但是无法匹配设备号
    # commandList=[]
    # for one in data['model'].split('\n'):
    #     commandList=commandList+modle2command_new(str(one))
    # commandStr=commandConstruct(deviceId=data['deviceId'], commandList=commandList)
    # commandStrList=commandStr.split('\n')
    # print(commandStrList)


    # pin.put_select(name='name',label='选择要发送得指令',options=commandStrList)
    pin.put_checkbox(name='name',label='选择要发送的单条指令',options=commandStrList)
    def sendRequest(content):
        content=pin.pin.name
        print(content)
        res=HttpOper().call('POST', data['url'],headers=json.loads(data['header']),data=content[0])
        res1=res.res.content.decode('utf-8')
        output.popup(title='请求响应信息',content=output.put_text(f"地址：\n{data['url']}\n请求头：\n{data['header']}\n请求数据：\n{content}\n响应：\n{res1}"))
    
    output.put_button(label="单条发送", onclick=lambda: sendRequest(pin.pin.num))


    def sendRequestAll(contentList):
        resList=[]
        for one in contentList:
            res=HttpOper().call('POST', data['url'],headers=json.loads(data['header']),data=one)
            res1=res.res.content.decode('utf-8')
            resList.append(res1+"\n"+"*"*20+"\n")
        
        output.popup(title='请求响应集合',content=output.put_text(f"响应集合：\n{''.join(resList)}"))
    
    output.put_html("<h1>全部发送<h1>")
    output.put_text("\n".join(commandStrList))
    output.put_button(label="全部发送", onclick=lambda: sendRequestAll(commandStrList))


if __name__=='__main__':
    start_server(businessProcess,port=9999,debug=True,cdn=False,auto_open_webbrowser=True)




import telnetlib
from loguru import logger

class TelnetClient(object):
    """通过telnet连接dubbo服务, 执行shell命令, 可用来调用dubbo接口
    """

    def __init__(self, server_host, server_port):
        self.conn = telnetlib.Telnet()
        self.server_host = server_host
        self.server_port = server_port

    # telnet登录主机
    def connect_dubbo(self):
        try:
            logger.info("telent连接dubbo服务端: telnet {} {} ……".format(self.server_host, self.server_port))
            self.conn.open(self.server_host, port=self.server_port)
            return True
        except Exception as e:
            logger.info('连接失败, 原因是: {}'.format(str(e)))
            return False

    # 执行传过来的命令，并输出其执行结果
    def execute_command(self, command):
        # 执行命令
        cmd = 'invoke {}\n'.format(command).encode("utf-8")
        self.conn.write(cmd)
        # 初始化调用次数
        invoke_count = 0
        # 若调用无返回时，记录次数并重试
        result = self.conn.read_very_eager().decode(encoding='utf-8').split('\r\n')[0]
        result = self.conn.read_very_eager().decode(encoding='utf-8')
        print("result is :"+result)
        while result == '':
            sleep(1)
            result = self.conn.read_very_eager().decode(encoding='utf-8').split('\r\n')[0]
            invoke_count += 1
            if invoke_count>=5:
                logger.info("调用dubbo接口超过五次，调用失败")
                return '调用dubbo接口失败'
        return result
    # 退出telnet
    def logout_host(self):
        self.conn.write(b"exit\n")
        logger.info("登出成功")

class telnet_exe(object):

    def __init__(self, content):
        #解析dubbo部署的ip和port
        try:
            dubboaddrre = re.compile(r"([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+:[0-9]+)", re.I)
            result = dubboaddrre.search(str(content)).group()
            server_host = result.split(":")[0]
            server_port = result.split(":")[1]
            logger.info("获取到dubbo部署信息" + result)
        except Exception as e:
            raise Exception("获取dubbo部署信息失败：{}".format(e))

        try:
            self.telnet_client = TelnetClient(server_host, server_port)
            self.login_flag = self.telnet_client.connect_dubbo()
        except Exception as e:
            logger.info("invokedubboapi init error" + e)

    def __enter__(self):
        return self

    #调用dubbo接口
    def invoke_dubbo_api(self, data):
        cmd = data.get("dubbo_service") + "." + data.get("dubbo_method") + "{}".format(data.get("parameters"))
        logger.info("调用命令是：invoke {}".format(cmd))
        resp = None
        try:
            if self.login_flag:
                result= self.telnet_client.execute_command(cmd)
                logger.info("接口响应是,result={}".format(result))
                return result
            else:
                logger.info("登陆失败！")
        except Exception as e:
            raise Exception("调用接口异常, 接口响应是result={}, 异常信息为：{}".format(result, e))
        # self.logout()

    # 调用多个dubbo接口，注：确保所有接口是同一个ip和port
    def invoke_dubbo_apis(self,datas):
        summary = []
        if isinstance(datas,list):
            for i in range(len(datas)):
                result = self.invoke_dubbo_api(datas[i])
                summary.append({"data":datas[i],"result":result})
            return summary
        else:
            return "请确认入参是list"

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.telnet_client.logout_host()


from loguru import logger
from pywebio.input import input, FLOAT,NUMBER,input_group,select
from pywebio.output import put_text
from pywebio import start_server
import os,time
from main import RemoteRunner,RunnerResult

def oneCheck():
    input_data={}
    go_on=True
    while go_on:
        server_data = input_group("禅道版本号创建向导",[
        input('产品编码，通常是英文或者拼音简称,如user_center', name='desc'),
        input('输入产品版本号，初次创建为V1.0.0', name='version'),
        ])
        desc=server_data['desc'].lower()
        version=server_data['version'].lower().strip('v')
        major,middle,miner=version.split('.')

        put_text(f'产品代号为：{desc}-V{major}00')
        put_text(f'产品计划名称为：{desc}-V{major}00R0{middle}')
        put_text(f'项目代号为：{desc}-V{major}00R0{middle}B0{miner}')

#docker run -itd -p 80:8080 6648f88023f4




if __name__ == '__main__':
    start_server(oneCheck, port=8080)






import datetime,time
import json
import threading
from kafka import KafkaConsumer,KafkaProducer
from kafka.structs import TopicPartition
from kafka.client_async import KafkaClient
from core.utils import isJson
import jmespath
import threading
from loguru import logger
import re
import websockets
import asyncio
from concurrent.futures import ThreadPoolExecutor
from collections import deque

logger.add('run_log.txt',encoding='utf-8')

class kafkaOper(object):
    '''
    main class for kafka general operator, which not tested if you are connect kafka with credentials
    '''
    def __init__(self,topic,bootstrapserver):
        self.topic=topic
        self.bootstrapserver=bootstrapserver
        self.kafkaConnection=None
    def getConsumer(self,clientid="kafkaOper",auto_offset_reset="latest",**args):
        '''
        get consumer
        :param clientid:
        :param auto_offset_reset:
        :param args:
        :return:
        '''
        self.kafkaConnection=KafkaConsumer(self.topic,bootstrap_servers=self.bootstrapserver,client_id=clientid,auto_offset_reset=auto_offset_reset)
        return self
    def getProducer(self,clientid="kafkaOper"):
        '''
        get producer
        :param clientid:
        :return:
        '''
        self.kafkaConnection=KafkaProducer(bootstrap_servers=self.bootstrapserver,client_id=clientid)
        return self
    def getSubscribe(self):
        '''
        get subscriber
        :return:
        '''
        self.kafkaConnection=KafkaConsumer(bootstrap_servers=self.bootstrapserver)
        return self
    def jsonFilter(self,allStr,pattern,key):
        '''
        filter string by json
        :param allStr: origianl string
        :param pattern: jmeshpath parttern str
        :param key: target key to match
        :return:
        '''
        if isJson(allStr.value.decode()):
            try:
            # print(flag)
                allStr = json.loads(allStr.value.decode())
                flag=jmespath.search(pattern,allStr)
                if flag==key:
                    logger.warning(f'Topic: {self.topic} 在时间：{datetime.datetime.now()}->结果匹配到: {key}了')
                    logger.info(f'Total String is: {allStr}')
                # else:
                #     logger.info(f'{allStr}')
            except Exception as e:
                logger.error('json 解析失败')
                logger.error(f'{allStr}')
        logger.error(f"{allStr} isn't a string.")

    def returnJsonFilter(self,allStr,pattern,key):
        '''
        filter string by json and return the result
        :param allStr:
        :param pattern:
        :param key:
        :return:
        '''
        if isJson(str(allStr.value.decode())):
            try:
                allStr = json.loads(allStr.value.decode())
                flag=jmespath.search(pattern,allStr)
                # print(f'flag: {flag}')
                # print(f'key: {key}')
                if flag in key:
                    return allStr
            except Exception as e:
                logger.error('json 解析失败')
                logger.error(f'{allStr}')
        else:
            logger.error(f"{allStr} isn't a string.")
            return False

    def returnRegxFilter(self,allStr,pattern, key):
        '''
        filter string by regx and return the string
        :param allStr: orignal string
        :param pattern: match parttern
        :param key: target str to match
        :return:
        '''
        try:
            allStr = allStr.value.decode()
            flag=re.compile(pattern)
            res=flag.findall(allStr)
            # print(f'res: {res}')
            # print(f'key: {key}')
            if key in "".join(res):
                return allStr
            # else:
            #     logger.info(f'{allStr}')
        except Exception as e:
            logger.error('regx 解析失败')
            logger.error(f'{allStr}') 
            return False

    def regxFilter(self,allStr,pattern, key):
        '''
        filter string by regx
        :param allStr:
        :param pattern:
        :param key:
        :return:
        '''
        try:
            allStr = allStr.value.decode()
            flag=re.compile(pattern)
            res=flag.findall(allStr)
            # print(res)
            if key in "".join(res):
                logger.warning(f'Topic: {self.topic} 在时间：{datetime.datetime.now()}->结果匹配到: {res}了')
                logger.info(f'Total String is: {allStr}')
            # else:
            #     logger.info(f'{allStr}')
        except Exception as e:
            logger.error('regx 解析失败')
            logger.error(f'{allStr}') 

    def doFileterFromComsumer(self,flag="",pattern="",matchStr="",keepListen=False,store=False):
        '''
        conbine the filter way in one function
        :param flag:
        :param pattern:
        :param matchStr:
        :param keepListen:
        :param store:
        :return:
        '''
        '''
            flag = "json"
            or
            flag ="regx"
        '''
        logger.info('Start filter....')
        resStore=[]
        for one in self.kafkaConnection:
            # print(one)
            # print(flag)
            if flag=="json":
                # logger.info(f"json {one}")
                self.jsonFilter(allStr=one,pattern=pattern,key=matchStr)
            elif flag=="regx":
                # logger.info(f"regx {one}")
                self.regxFilter(allStr=one,pattern=pattern,key=matchStr)
            elif flag=="":
                logger.warning(f'Topic: {self.topic} 在时间：{datetime.datetime.now()}->未加入匹配的消息是: {one}了')
                
    def doSendFromProducer(self,message,partition=None,key=None):
        '''
        send kafka messeage
        :param message:
        :param partition:
        :param key:
        :return:
        '''
        self.kafkaConnection.send(self.topic,value=bytes(message,encoding='utf-8'),partition=partition,key=key)

    def retrivalFixedMsg(self,interval_ms,getNum):
        '''
        get fixed number message from topic
        :param interval_ms:
        :param getNum:
        :return:
        '''
        self.kafkaConnection.subscribe(self.topic)
        # print(self.kafkaConnection.poll(timeout_ms=interval_ms,max_records=getNum,update_offsets=False).items())
        # return self.kafkaConnection.poll(timeout_ms=interval_ms,max_records=getNum,update_offsets=False).items()
        resList=[]
        while len(resList)<getNum:
            for k,v in self.kafkaConnection.poll(timeout_ms=interval_ms,max_records=getNum,update_offsets=True).items():
                # print(f"{k}--->{v}")
                for one in v:
                    # print(f"data is : {one}")
                    resList.append(one.value.decode())
            # print(len(resList))
        return resList

    def retrivalFixedMsgWithFilter(self,interval_ms,getNum,filterFlag,pattern,matchStr):
        '''
        get fixed number of message from topic with filter
        :param interval_ms:
        :param getNum:
        :param filterFlag:
        :param pattern:
        :param matchStr:
        :return:
        '''
        self.kafkaConnection.subscribe(self.topic)
        # print(self.kafkaConnection.poll(timeout_ms=interval_ms,max_records=getNum,update_offsets=False).items())
        # return self.kafkaConnection.poll(timeout_ms=interval_ms,max_records=getNum,update_offsets=False).items()
        resList=[]
        while len(resList)<getNum:
            for k,v in self.kafkaConnection.poll(timeout_ms=interval_ms,max_records=getNum,update_offsets=True).items():
                # print(f"{k}--->{v}")
                for one in v:
                    print(f"data is : {one}")
                    if filterFlag=='json':
                        try:
                            print(filterFlag)
                            allStr = json.loads(one.value.decode())
                            flag=jmespath.search(pattern,allStr)
                            if flag==matchStr:
                                print(f'got json:::{allStr}')
                                resList.append(one.value.decode())
                        except Exception as e:
                            logger.error('json 解析失败')
                            logger.error(f'{allStr}')
                    elif filterFlag=='regx':
                        try:
                            allStr = one.value.decode()
                            flag=re.compile(pattern)
                            res=flag.findall(allStr)
                            if matchStr in "".join(res):
                                resList.append(one.value.decode())
                        except Exception as e:
                            logger.error('regx 解析失败')
                            logger.error(f'{allStr}') 
            # print(len(resList))
        return resList
    
    def retrivalFlowMsg(self,flag,pattern,key):
        '''
        continue get message from topic with filter
        :param flag:
        :param pattern:
        :param key:
        :return:
        '''
        self.kafkaConnection.subscribe(self.topic)
        while True:
        # now = datetime.datetime.utcnow().isoformat() + 'Z'
            msg=self.kafkaConnection.poll(timeout_ms=0,max_records=10)
            for k,v in msg.items():
                for one in v:
                    if flag=="json":
                        yield self.returnJsonFilter(one, pattern, key)
                    elif flag=="regx":
                        yield self.returnRegxFilter(one, pattern, key)
                    else:
                        yield one.value.decode()
    
    def getStartFromTimeStamp(self,minutes=1,flag="",pattern="",key=""):
        '''search from timestamp_ms, minuts 1 means 1 mins ago'''
        print('in getFromTimeStamp')
        resList=[]
        timestamp_ms=round(datetime.datetime.timestamp(datetime.datetime.now()+datetime.timedelta(minutes=-minutes))*1000)
        print(timestamp_ms)
        try:
            # topic = 'my_favorite_topic'
            # bootstrap_servers = 'localhost:9092'
            # consumer = KafkaConsumer(topic=self.topic, bootstrap_servers=self.bootstrapserver, auto_offset_reset='earliest')
            self.getConsumer(auto_offset_reset='earliest')
            # 通过主题获取分区集
            partition_set = self.kafkaConnection.partitions_for_topic(self.topic)
            # 通过主题和分区根据时间戳获取相应的偏移量
            offsets = self.kafkaConnection.offsets_for_times({TopicPartition(self.topic, partition): timestamp_ms for partition in partition_set})
            # 手动偏移
            for partition, oat in offsets.items():
                self.kafkaConnection.seek(partition, oat.offset)

            # return consumer
            for msg in self.kafkaConnection:
                (res,origStr)=self.checkFromTimeStamp(msg, flag, pattern, key)
                if res==True:
                    self.kafkaConnection.close()
                    return True, origStr
                else:
                    return False, origStr 
                # print(msg.value.decode())
                # if searchStr in msg.value.decode():
                #     print(msg.value.decode())
                #     self.kafkaConnection.close()
                #     return True
                # yield msg.value.decode()
            #     resList.append(msg.value.decode())
            #     print(resList)
            # return resList
        except AssertionError:
            # Unassigned partition
            self.kafkaConnection.close()
            return seek_offsets_for_timestamp(timestamp)

    def checkFromTimeStamp(self,one,flag,pattern,key):
        '''check data from timestamp'''
        print('in checkFromTimeStamp')
        if flag=="json":
            if self.returnJsonFilter(one, pattern, key)!=False:
                return True, one.value.decode()
            else:
                return False, one.value.decode()
        elif flag=="regx":
            if self.returnRegxFilter(one, pattern, key)!=False:
                return True, one.value.decode()
            else:
                return False, one.value.decode()
        elif flag=="contain":
            if key in one.value.decode():
                return True, one.value.decode()

    def get_offset_time_window(self, begin_time, end_time):
        partitions_structs = []
        
        for partition_id in self.kafkaConnection.partitions_for_topic(self.topic):
            partitions_structs.append(TopicPartition(self.topic, partition_id))

        begin_search = {}
        for partition in partitions_structs:
            begin_search[partition] = begin_time if isinstance(begin_time, int) else self.__str_to_timestamp(begin_time)
        begin_offset = self.kafkaConnection.offsets_for_times(begin_search)

        end_search = {}
        for partition in partitions_structs:
            end_search[partition] = end_time if isinstance(end_time, int) else self.__str_to_timestamp(end_time)
        end_offset = self.kafkaConnection.offsets_for_times(end_search)

        for topic_partition, offset_and_timestamp in begin_offset.items():
            b_offset = 'null' if offset_and_timestamp is None else offset_and_timestamp[0]
            e_offset = 'null' if end_offset[topic_partition] is None else end_offset[topic_partition][0]
            print('Between {0} and {1}, {2} offset range = [{3}, {4}]'.format(begin_time, end_time, topic_partition,
                                                                              b_offset, e_offset))
        return topic_partition,b_offset, e_offset

    @staticmethod
    def __str_to_timestamp(timeStr, format_style='%Y-%m-%d %H:%M:%S'):
        # time_array = time.strptime(str_time, format_type)
        # return int(time.mktime(time_array)) * 1000
        datetime_obj = datetime.strptime(timeStr, format_style)
        ret_stamp = int(time.mktime(datetime_obj.timetuple()) * 1000.0 + datetime_obj.microsecond / 1000.0)
        return ret_stamp

    def kafkaMsgCompareBettwenTimeByKey(self,startTime:str,endTime:str):
        '''
        :param kafkaComsumer:
        :param startTime: '2022-03-31 19:27:05.000'
        :param endTime:'2022-03-31 19:28:33.999'
        :return:res: dict string
        '''
        from collections import deque
        res={}
        # k_in = kafkaOper(topic='tacos-kernel-driver-topic', bootstrapserver=host).getConsumer()
        par, start, end = self.kafkaConnection.get_offset_time_window(begin_time=startTime, end_time=endTime)
        self.kafkaConnection.kafkaConnection.seek(par, start)

        for msg in self.kafkaConnection.kafkaConnection:
            id=json.loads(msg.value.decode().replace("\n","").replace("\t",""))['id']
            q = deque()
            if msg.offset >= end:
                break
            else:
                if id not in res.keys():
                    q.append(msg.timestamp)
                    res[id]=q
                else:
                    q.append(msg.timestamp)
                    res[id].extend(q)
        return res


def general_orderMsg(topic,serverAndPort,interval_ms,getNum,callbackFlag=False,callbackFuc=None):
    if callbackFlag==False:
        return kafkaOper(topic,serverAndPort).getSubscribe().retrivalFixedMsg(interval_ms,getNum)
    else:
        callbackFuc(kafkaOper(topic,serverAndPort).getSubscribe().retrivalFixedMsg(interval_ms,getNum))


def general_orderMsgWithFilter(topic,serverAndPort,interval_ms,getNum,filterFlag,pattern,matchStr,callbackFlag=False,callbackFuc=None):
    if callbackFlag==False:
        return kafkaOper(topic,serverAndPort).getSubscribe().retrivalFixedMsgWithFilter(interval_ms,getNum,filterFlag,pattern,matchStr)
    else:
        callbackFuc(kafkaOper(topic,serverAndPort).getSubscribe().retrivalFixedMsgWithFilter(interval_ms,getNum,filterFlag,pattern,matchStr))

def continue_orderMsg(topic,serverAndPort,flag,pattern,key):
    yield kafkaOper(topic,serverAndPort).getSubscribe().retrivalFlowMsg(flag,pattern,key)


def continue_orderMsg2(topic,serverAndPort,flag,pattern,key):
    print(f'{topic},{serverAndPort},{flag},{pattern},{key}')
    yield kafkaOper(topic,serverAndPort).getSubscribe().retrivalFlowMsg(flag,pattern,key)


def continue_orderMultiMsg(topics=[],serverAndPorts=[],flags=[],patterns=[],keys=[]):

    if type(topics)!=list or type(serverAndPorts)!=list or type(flags)!=list or type(patterns)!=list or type(keys)!=list:
        return f'input not the type of list'

    pool=ThreadPoolExecutor(len(topics))
    all=[pool.map(continue_orderMsg2,a,b,c,d,e) for a,b,c,d,e in zip(topics,serverAndPorts,flags,patterns,keys)]
    # print(len(all))
    for ttt in all:
        print(f"ttt {ttt}")
        yield ttt






def general_sender(topic="",serverAndPort="localhost:9092",message=""):
    kafkaOper(topic,serverAndPort).getProducer().doSendFromProducer(message)

def general_listener(topic="",serverAndPort="localhost:9092",flag="",pattern="",key=""):
    kafkaOper(topic,serverAndPort).getConsumer().doFileterFromComsumer(flag,pattern,key)

def multi_topic_listener(topicList=[],serverAndPortList=[],flagList=[],patternList=[],keyList=[]):
    threadList=[]
    for one in zip(topicList,serverAndPortList,flagList,patternList,keyList):
        (a,b,c,d,e)=one
        # print(f"{a},{b},{c},{d}")
        temp=threading.Thread(target=general_listener,args=(a,b,c,d,e))
        temp.start()
        threadList.append(temp)
    for one in threadList:
        one.join()




import asyncio
import websockets
import kafka
import random
from functools import partial

async def kafkaMsgRecv(websocket=None, serverPath='0.0.0.0:9092',topic="testTopic",interval=0,nums=None):
    k_conn=kafka.KafkaConsumer(bootstrap_servers=serverPath)
    k_conn.subscribe(topic)
    while True:
        # now = datetime.datetime.utcnow().isoformat() + 'Z'
        msg=k_conn.poll(timeout_ms=interval,max_records=nums)
        for k,v in msg.items():
            for one in v:
                # print(one.value.decode())
                await websocket.send(one.value.decode())
                await asyncio.sleep(random.random() * 3)


async def kafkaMsgRecvWithFilter(websocket=None,flag="",pattern="",key="", serverPath='0.0.0.0:9092', topic="testTopic", interval=0, nums=None):
    for one in continue_orderMsg(flag=flag,pattern=pattern,key=key,topic=topic,serverAndPort=serverPath):
        for a in one:
            # print(f'a is:{a}')
            if a is not None:
                print(a)
                try:
                    await websocket.send(json.dumps(a,ensure_ascii=False))
                    await asyncio.sleep(0)
                except Exception as e:
                    print(e)
                    print(a)

async def muiltiKafkaMsgRecvWithFilter(websocket=None,flag="",pattern="",key="", serverPath='0.0.0.0:9092', topic="testTopic", interval=0, nums=None):
    for one in continue_orderMsg(flag=flag,pattern=pattern,key=key,topic=topic,serverAndPort=serverPath):
        for a in one:
            # print(f'a is:{a}')
            if a is not None:
                try:
                    await websocket.send(json.dumps(a))
                    await asyncio.sleep(0)
                except Exception as e:
                    print(e)
                    print(a)

def multi_topic_listener(topicList=[],serverAndPortList=[],flagList=[],patternList=[],keyList=[]):
    threadList=[]
    for one in zip(topicList,serverAndPortList,flagList,patternList,keyList):
        (a,b,c,d,e)=one
        # print(f"{a},{b},{c},{d}")
        temp=threading.Thread(target=general_listener,args=(a,b,c,d,e))
        temp.start()
        threadList.append(temp)
    for one in threadList:
        one.join()



def kafkaFetchServer(interval=0,nums=None,serverPath='0.0.0.0:9092',topic="testTopic",local='0.0.0.0',port=8765):
    # print(serverPath)
    newloop=asyncio.new_event_loop()
    asyncio.set_event_loop(newloop)
    loop=asyncio.get_event_loop()
    start_server = websockets.serve(partial(kafkaMsgRecv,serverPath=serverPath,topic=topic), local, port)
    loop.run_until_complete(start_server)
    loop.run_forever()

def kafkaFetchServerWithFilter(interval=0,nums=None,serverPath='0.0.0.0:9092',topic="testTopic",flag="",pattern="",key="",local='0.0.0.0',port=8765):
    # print(serverPath)
    print('in kafka fetch server with filter:'.format(flag,pattern,key,serverPath,topic))
    newloop=asyncio.new_event_loop()
    asyncio.set_event_loop(newloop)
    loop=asyncio.get_event_loop()
    start_server = websockets.serve(
        partial(kafkaMsgRecvWithFilter, flag=flag, pattern=pattern, key=key, serverPath=serverPath, topic=topic), local,port)
    loop.run_until_complete(start_server)
    loop.run_forever()


if __name__ == '__main__':
    # multi_topic_listener(topicList=['topic','topic1','topic2'],serverAndPortList=["ip:port","ip:port","ip:port"]
    # ,patternList=["json.data.id","json.data.id","json.data.id"],keyList=["assertingData1","assertingData2","assertingData3"])


    # multi_topic_listener(topicList=['testTopic'],
    # serverAndPortList=["192.168.xxx.xxx:9092"],
    # flagList=["regx"],
    # partternList=["abb(.*)bba"],
    # keyList=["555"])


    # multi_topic_listener(topicList=['iotHub','device_default_prop','device_default_state'],
    # serverAndPortList=["192.168.xxx.xxx:9092","192.168.xxx.145:9092","192.168.xxx.xxx:9092"],
    # flagList=["json","json","json"],
    # patternList=["payload.virDevUid","payload.virDevUid","payload.deviceId"],
    # keyList=["914959009603264531","914959009603264531","914959009603264536"])

    
    # general_listener(topic="testTopic",serverAndPort="0.0.0.0:9092",flag="",pattern="",key="")


    # kafkaFetchServer(0,None,'192.168.2.101:9092','testTopic')
    # kafkaFetchServerWithFilter(0,None,'192.168.xxx.xxx:9092','testTopic','regx',"(\d{4}-\d{02}-\d{02})",'2022-01-04')
    # kafkaFetchServerWithFilter(0, None, '192.168.2.101:9092', 'testTopic', '', "",'')

    # for one in continue_orderMsg('aaa','192.168.2.101:9092','',"",''):
    #     for a in one:
    #         print(a)



    # for one in continue_orderMultiMsg(['bbb','ccc'], ['192.168.2.101:9092','192.168.2.101:9092'], ["",""], ["",""], ["",""]):
    #     print(f"one {one}")
    #     for m in one:
    #         print(f"mmm {m}")
    #         for i in t:
    #             print(f"iii {i}")



    # general_listener(topic="testTopic",serverAndPort="192.168.xxx.xxx:9092",flag="regx",pattern=r"abb(.*)bba",key="555")
    # general_sender(topic="testTopic",serverAndPort="192.168.xxx.xxx:9092",message='{"abc":{"bcd":"555"}}')
    
    # import time
    # k=kafkaOper(topic='test2',bootstrapserver='127.0.0.1:9092')
    # print(k.getFromTimeStamp(1, flag="contain", pattern="", key="data"))

    k=kafkaOper(topic='test0330', bootstrapserver='192.168.2.103:9092').getConsumer()
    par, start, end=k.get_offset_time_window(begin_time='2022-03-30 22:30:00', end_time='2022-03-30 22:35:00')
    print(par)
    print(type(par))
    print(start)
    print(end)
    # k.kafkaConnection.assign([par])
    k.kafkaConnection.seek(par, start)
    for msg in k.kafkaConnection:
        if msg.offset > end:
            break
        else:
            print(msg.timestamp,msg.value)
    # while True:
    #     try:
    #         value_ans = k.kafkaConnection.poll(max_records=20).values()
    #         if value_ans>0:
    #             for par in value_ans:
    #                 msg_offset=int(par.offset)
    #                 msg_partition=int(par.partition)
    #                 msg_topic = str(par.topic)
    #                 if par.offset==end:
    #                     break;
    #                 message_sets += par
    #     except Exception as e:
    #         print(e)


import setuptools
import platform



with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="khandytool",
    version="0.2.72",
    author="Ou Peng",
    author_email="kevin72500@qq.com",
    description="khandytool, handy core in testing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kevin72500/khandytool",
    packages=setuptools.find_packages(),
    package_data={'core.bladeTest': ['xmindStructure.jpg']},
    include_package_data=True,
    install_requires=['jsonpath==0.82','xmltodict==0.13.0','flashtext','fabric==2.6.0','pytest==6.2.5','pywebio==1.6.0','requests==2.26.0','loguru==0.5.3','jinja2==3.0.2','openpyxl==3.0.9','xmindparser==1.0.9','jmespath==0.10.0','pymysql==1.0.2','swaggerjmx==1.0.9','faker==8.12.1','websockets==10.1','kafka-python==2.0.2','flask==2.0.2','Werkzeug==2.0.2','paho-mqtt==1.5.1','msgpack==1.0.3','redis==4.1.3','python-dotenv==0.19.2','jsonschema',"pycryptodome==3.12.0; platform_system=='Windows'","pycrypto==2.6.1; platform_system=='Linux'","pycrypto==2.6.1; platform_system=='Darwin'","tenacity","wget","pyttsx3","pyyaml","xlrd","pytest-html"],
    # data_files=[
    #     ('jmx',["apache-jmeter-5.4.1.zip"]),
    # ],
    # extras_requires={
    #     "pycryptodome==3.12.0; sys_platform=='Windows'",
    #     "pycrypto==2.6; sys_platform=='Linux'",
    #     "pycrypto==2.6; sys_platform=='Darwin'"
    # },
    entry_points={
        'console_scripts': [
            'khandytool=khandytool:core',
            'toolrun=core:run',
            'kframe=kframe:kframe'

            # 'khandytool=khandytool',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)

